/* Utilitaires de lecture pour les fichiers PPM
 *
 *
 * E.B. 12/98
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "pm_util.h" 

void pm_erreur(char *texte)
{ 
  fprintf(stderr, "\n%s \n\n", texte);
  exit(1);
}

char pm_getc(FILE* file)
    {
    register int ich;
    register char ch;
 
    ich = getc( file );
    if ( ich == EOF )
        pm_erreur("EOF / read error" );
    ch = (char) ich;
    
    if ( ch == '#' )
        {
        do
            {
            ich = getc( file );
            if ( ich == EOF )
                pm_erreur("EOF / read error" );
            ch = (char) ich;
            }
        while ( ch != '\n' && ch != '\r' );
        }
 
    return ch;
    }


bit pm_getbit(FILE* file)
    {
    register char ch;
 
    do
        {
        ch = pm_getc( file );
        }
    while ( ch == ' ' || ch == '\t' || ch == '\n' || ch == '\r' );
 
    if ( ch != '0' && ch != '1' )
        pm_erreur("junk in file where bits should be" );
 
    return ( ch == '1' ) ? 1 : 0;
    }
 
 
unsigned char pm_getrawbyte(FILE* file)
    {
    register int iby;
 
    iby = getc( file );
    if ( iby == EOF )
        pm_erreur("EOF / read error" );
    return (unsigned char) iby;
    }
 
int pm_getint( FILE* file)
    {
    register char ch;
    register int i;
 
    do
        {
        ch = pm_getc( file );
        }
    while ( ch == ' ' || ch == '\t' || ch == '\n' || ch == '\r' );
 
    if ( ch < '0' || ch > '9' )
        pm_erreur( "junk in file where an integer should be" );
 
    i = 0;
    do
        {
        i = i * 10 + ch - '0';
        ch = pm_getc( file );
        }
    while ( ch >= '0' && ch <= '9' );
 
    return i;
    }



/* Lecture du fichier d'image */
unsigned char *LoadPPMImage( char *file, int *ImgW, int *ImgH, int *maxval)
{
  FILE* ifp;
  unsigned char* pixmap;
  int ich1, ich2 ;
  int ppmraw ;
  int i,j,k ;
 
  /* Ouverture */
  if(strcmp(file,"-") == 0)
    ifp = stdin;
  else if((ifp = fopen(file,"r"))==0)
    pm_erreur("Error reading file ");
 
   
  /* Lecture du Magic number */
  ich1 = getc( ifp );
  if ( ich1 == EOF )
    pm_erreur( "EOF / read error reading magic number" );

  ich2 = getc( ifp );
  if ( ich2 == EOF )
    pm_erreur( "EOF / read error reading magic number" );

  switch(ich2){
  case '3': 
    ppmraw = 0;
    break;
  case '6': 
    ppmraw = 1;
    break;
  default:
    pm_erreur(" wrong file format ");
  }
 
  /* Lecture des dimensions */
  *ImgW = pm_getint( ifp );
  *ImgH = pm_getint( ifp );
  *maxval = pm_getint( ifp );

  
  /* Allocation memoire  */
  pixmap = (unsigned char*) malloc( 3 * *ImgW * *ImgH * 
                                    sizeof(unsigned char));
  /* Lecture */
  for(i=0; i < *ImgH; i++)
    for(j=0; j < *ImgW ; j++)
 
      /* ppm raw */
      if(ppmraw){
        for(k=0; k < 3 ; k++)
          pixmap[((*ImgH - i - 1) * *ImgW + j) * 3 + k] = 
            pm_getrawbyte(ifp) ;
      }
  
      /* ppm ascii */
      else{
        for(k=0; k < 3 ; k++)
          pixmap[((*ImgH - i - 1) * *ImgW + j) * 3 + k] = pm_getint(ifp);
      }  
  
  return pixmap ;
}
/* Lecture du fichier d'image */
unsigned char *LoadPGMImage( char *file, int *ImgW, int *ImgH, int *maxval)
{
  FILE* ifp;
  unsigned char* pixmap;
  int ich1, ich2 ;
  int pgmraw ;
  int i,j;
 
  /* Ouverture */
  if(strcmp(file,"-") == 0)
    ifp = stdin;
  else if((ifp = fopen(file,"r"))==0)
    pm_erreur("Error reading file ");
 
   
  /* Lecture du Magic number */
  ich1 = getc( ifp );
  if ( ich1 == EOF )
    pm_erreur( "EOF / read error reading magic number" );

  ich2 = getc( ifp );
  if ( ich2 == EOF )
    pm_erreur( "EOF / read error reading magic number" );

  switch(ich2){
  case '2': 
    pgmraw = 0;
    break;
  case '5': 
    pgmraw = 1;
    break;
  default:
    pm_erreur(" wrong file format ");
  }
 
  /* Lecture des dimensions */
  *ImgW = pm_getint( ifp );
  *ImgH = pm_getint( ifp );
  *maxval = pm_getint( ifp );

  
  /* Allocation memoire  */
  pixmap = (unsigned char*) malloc(  *ImgW * *ImgH * 
                                    sizeof(unsigned char));
  /* Lecture */
  for(i=0; i < *ImgH; i++)
    for(j=0; j < *ImgW ; j++)
 
      /* pgm raw */
      if(pgmraw)
          pixmap[(*ImgH - i - 1) * *ImgW + j] = pm_getrawbyte(ifp) ;

  
      /* pgm ascii */
      else
          pixmap[(*ImgH - i - 1) * *ImgW + j] = pm_getint(ifp);
        
  
  return pixmap ;
}

/*Ecriture */
void WritePPMImage(int ImgW, int ImgH, int MaxVal, unsigned char *Image)
{
  int i,j,k;

  printf("P6\n");
  printf("%d %d\n", ImgW, ImgH);
  printf("%d\n",MaxVal);
  for(i=0; i < ImgH ; i++)
    for(j=0; j < ImgW ; j++)
      for(k=0; k < 3 ; k++)
	printf("%c", Image[((ImgH - i - 1)  * ImgW + j) * 3 + k]);
  



}
void WritePGMImage(int ImgW, int ImgH, int MaxVal, unsigned char *Image)
{
  int i,j;

  printf("P5\n");
  printf("%d %d\n", ImgW, ImgH);
  printf("%d\n",MaxVal);
  for(i=0; i < ImgH ; i++)
    for(j=0; j < ImgW ; j++)
	printf("%c", Image[(ImgH - i - 1)  * ImgW + j]);
  



}


